/*------------------------------------------------------------------------------
#
#	TV-Man.Sound.h
#
#	Copyright  Apple Computer, Inc. 1989-1990
#	All rights reserved.
#
#
#
#
#	This is the file that is common to all Sound stuff.
#
#	In order to have an evironment which predictable events happen several definitions
#	must be established. The most critical is in the file architecture. Each functional
#	block will have its own source and header file. The main project file,  in this
#	case TV-Man, will have its header file included with all other files. This will
#	allow for global constants. The utility source file shall contain functions that
#	are general purpose in nature and that can be used by all other functions. These
#	are intended not to be application or major block specific. In order for this to be 
#	accomodated all functions in the utility source file must use only the information
#	that is passed to them or information that can be gleaned from the system via
#	toolbox calls. There will be no header file associated with the utility file as this
#	will destroy the intent of the utilities.
#
#	There are several files which contain information which is global in nature .These
#	file are included in the main project header file. They are: x.Errors.h, x.Ext.h,
#	x.Protos.h, x.Menus.h. The reason for containing them in seperate files is one of
#	convienience and accesability. 
#
#
#	Revision Log:
#	
#		4-26-91		RGK		Creation
#
#
------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/*	This is a block of resource IDs for the Sound Control system	*/

#define	rBeepMenu		(rSounds * 10)			/*	the Beep types Menu		*/
#define rControl		((rSounds * 10) + 1)	/*	control root constant	*/
#define rBeepData		((rSounds * 10) + 2)	/*	Beep Data root constant	*/
#define rSoundIcon		((rSounds * 10) + 3)	/*	Icon root constant		*/





/*----------------------------------------------------------------------------*/
/*	The following constants are the resource ID numbers for the actual sounds.
	The SoundRsrc is a constant which will be used here and in the source code
	to develop the resource ID values from the Pop Up Menu values. */

#define rSineSnd		(rBeepMenu * 10) + 1	
#define rSquareSnd		(rBeepMenu * 10) + 2
#define rTriangleSnd	(rBeepMenu * 10) + 3
#define rSawtoothPosSnd	(rBeepMenu * 10) + 4
#define rSawtoothNegSnd	(rBeepMenu * 10) + 5





/*----------------------------------------------------------------------------*/
/*	The following constants calculate the control resource id numbers	*/

#define	rSndType		rControl * 10			/* Sound Type Control		*/
#define rBeepEnable		(rControl * 10) + 1		/* Beep Enable checkbox		*/
#define	rVolume			(rControl * 10) + 2		/* Sound Volume Control		*/
#define	rFreq			(rControl * 10) + 3		/* Sound Pitch Control		*/
#define	rTestBeep		(rControl * 10) + 4		/* Test Beep Button			*/
#define	rCancel			(rControl * 10) + 5		/* Cancel Button			*/
#define rBeepDone		(rControl * 10) + 6		/* Done Button				*/




/*----------------------------------------------------------------------------*/
/*	The following constants are the resource ID numbers for the Beep channel
	packages.	*/
	
#define rSopranoBData	rBeepData * 10
#define rAltoBData		(rBeepData * 10) + 1
#define rTenorBData		(rBeepData * 10) + 2
#define rBassBData		(rBeepData * 10) + 3
#define	rTestBData		(rBeepData * 10) + 4





/*----------------------------------------------------------------------------*/
/*	The following constants are the resource ID numbers for the Beep Dialog box
	Icons.	*/

#define rSpeaker		rSoundIcon * 10






/*----------------------------------------------------------------------------*/
/*	These are the item id numbers for sound generation dialog box. This group
	of constants must stay in coincidence with the resource dialog item list.	*/

#define BeepEnableId		1		/*	true to enable the sound generation	*/
#define BeepTypeId			4		/*	type of the beep (sine, square ect.)*/
#define BeepLengthId		7		/*	the length of the beep				*/
#define BeepQTimeId			10		/*	the length of time inbetween beeps	*/
#define BeepCountId			12		/*	the number of beeps					*/
#define BeepDelayId			15		/*	the number of beeps					*/
#define BeepVolumeCntlId	17		/*	the beep volume control				*/
#define BeepVolumeId		18		/*	the beep volume display				*/
#define BeepFreqCntlId		24		/*	the beep frequency control			*/
#define BeepFreqId			25		/*	the beep frequency display			*/
#define TestBeepId			30		/*	The test the Beep button			*/
#define BeepCancelId		31		/*	the Cancel button					*/
#define BeepDoneId			32		/*	the beep is done button				*/
#define DoneRingId			33		/*	the ring around the done button		*/
#define BeepTitleId			34		/*	the title of the dialog box			*/										
#define BeepIconId			35		/*	the is the Beep on? Icon			*/										
#define BeepMessageId		36		/*	the Icon Message					*/										






/*----------------------------------------------------------------------------*/
/*	These are the items for the Beep type menu.	*/

#define iSine				1
#define iSquare				2
#define iTriangle			3
#define iSawtoothPos		4
#define iSawtoothNeg		5




/*----------------------------------------------------------------------------*/
/*	These are the misc. constants for sound generation.	*/

#define LeftMargin		15		/*	The left margin in the Dialog text boxes	*/
#define RightMargin		5		/*	The right margin in the Dialog text boxes	*/
#define BottomMargin	4		/*	The bottom margin in the Dialog text boxes	*/
#define JumpCount		10		/*	The number of page jumps in the scroll bar	*/	
#define SndVerNumber	0		/*  The current version of the sound stuff 		*/




/*----------------------------------------------------------------------------*/
/*	These are the limits for the various sound edit boxes not found in the
	actual controls.	*/

#define LengthMax		32767
#define LengthMin		1
#define QTimeMax		32767
#define QTimeMin		0
#define CountMax		40
#define CountMin		1
#define	DelayMax		32767
#define	DelayMin		0





/*----------------------------------------------------------------------------*/
/*	The following line will effectivly eliminate the following typedefs from the 
	this header file during the Rez process because Rez cannot handle typedefs.
	All Resource files will define "__Rezing__" when processing, all C sources
	will not.	*/
	
#ifndef __Rezing__



#ifndef __Beep__	/* test to see if the Beep structure has already been defined	*/
#define	__Beep__			

/*----------------------------------------------------------------------------*/
/*	This is the resource structure for the beep data. Since it is used in TV-Man 
	as a global it is defined in the TV-Man.h file. This allows everyone to see
	it. The ifdef switch is used to limit the number of the Beep structures that 
	the compilier will try to use. This structure resides here because it is part
	of the sound system and if removed from TV-Man it will have to leave with it. */
	
typedef struct Beep
{
	short			Enabled;
	short			Type;
	short			Length;
	short			QTime;
	short			Count;
	short			Delay;
	short			Volume;
	short			Freq;
	short			Init;			/* the sound channel init id		*/
	short			RsrcId;			/* the resource id					*/
	Str32			Title;
} Beep, *BeepPtr, **BeepHdl;



/*----------------------------------------------------------------------------*/
/*	This Structure is used to hold the global varables which make a "Beep
	Channel".	*/
	
typedef struct BeepChan
{
	Boolean			Error;			/* true if any sound setup errored	*/
	Boolean			InProcess;		/* true if any sound setup errored	*/
	short			RsrcId;			/* the resource id					*/
	SndChannelPtr	SndChannel;		/* Pointer to the 1st sound channel	*/
} BeepChan, *BeepCPtr, **BeepCHdl;



#endif __Beep__
#endif __Rezing__

